
# dcUtil.py
#    Utility methods for CAF batch monitoring
#
# [Mark Neubauer 7/15/03]


# Standard modules
import os
import time

# My modules
import shared            #pass module for sharing objects between modules
from dcParams import *


# Define stdout and stderr write methods
#

def stdoutMessage(msg):
    os.write(1, "(%s): %s\n" % (time.ctime(time.time()), msg))

def stderrMessage(msg):
    os.write(2, "(%s): ERROR: %s\n" % (time.ctime(time.time()), msg))


# Write list of data to file. Which file depends upon type of data written
#

def writeData(dataID, dataList):

    firstElement = 1
    for dataElement in dataList:
        if firstElement:
            firstElement = 0
            dataStr = "%s" % dataElement
        else:
            dataStr += ",%s" % dataElement
    dataStr += "\n"

    if dataID == "pending":
        fd = os.open(DATA_PENDING_FILE,
                     os.O_WRONLY | os.O_CREAT | os.O_APPEND)
    elif dataID == "running":
        fd = os.open(DATA_RUNNING_FILE,
                     os.O_WRONLY | os.O_CREAT | os.O_APPEND)
    elif dataID == "wait_time":
        fd = os.open(DATA_WAIT_TIME_FILE,
                     os.O_WRONLY | os.O_CREAT | os.O_APPEND)
    elif dataID == "start_time_diff":
        fd = os.open(DATA_START_TIME_DIFF_FILE,
                     os.O_WRONLY | os.O_CREAT | os.O_APPEND)
    elif dataID == "exec_time":
        fd = os.open(DATA_EXEC_TIME_FILE,
                     os.O_WRONLY | os.O_CREAT | os.O_APPEND)
    else:
        stderrMessage("Invalid dataID = %s passed to writeData()" % dataID)
        return -1
    
    try:
        os.write(fd, "%s#%s" % (time.time(), dataStr))
    except OSError:
        stderrMessage("error in writing %s data file" % dataID)

    os.close(fd)


# Write list of data to file. Which file depends upon type of data written
#

def writeDataUser(userName, dataList):

    firstElement = 1
    for dataElement in dataList:
        if firstElement:
            firstElement = 0
            dataStr = "%s" % dataElement
        else:
            dataStr += ",%s" % dataElement
    dataStr += "\n"

    fileName = "user_%s.dat" % userName
    fileName = os.path.join(DATA_ROOT_DIR, fileName)
    fd = os.open(fileName, os.O_WRONLY | os.O_CREAT | os.O_APPEND)

    try:
        os.write(fd, "%s#%s" % (time.time(), dataStr))
    except OSError:
        stderrMessage("error in writing %s data file" % dataID)

    os.close(fd)

# Calculate average of entries in a given list
#

def getAvg(list):

    sum = 0.0
    if len(list) < 1:
        return None
    
    for item in list:
        sum += float(item)

    avg = sum / float(len(list))

    return avg
