
# Standard modules
import string

# FBSNG modules
from FBS_API import FBSClient

# My modules
from dcParams import *
from dcUtil import *
from class_jobTimes import jobTimes
from class_processInfo import processInfo


class CafInfo:

    # Contructor
    #

    def __init__(self):
        self.fc = FBSClient()
        self.processDict = {}
        self.jobTimesObj = jobTimes()


    #
    #

    def refresh(self):
        self.processDict = {}
        self.jobTimesObj.clear()
        sectionIDList = self.get()
        return sectionIDList
    

    # 
    #
    def get(self):
        sectionIDList = []
        for qname in self.fc.getQueueList():
            QInfo = self.fc.getQueue(qname)
            for sectionID in QInfo.Sections: #Note: sectionID = JID.sectionName
                try:
                    SectionInfo = self.fc.getSection(sectionID)
                    self.addProcesses(SectionInfo)
                    sectionIDList.append(sectionID)
                except KeyError:
                    stderrMessage("SectionID %s not found" % sectionID)

        return sectionIDList


    #
    #
    
    def addProcesses(self, SectionInfo):

        JID, sectionName = (SectionInfo.JobID, SectionInfo.Name)
        sectionID   = JID+"."+sectionName
        procType    = SectionInfo.ProcType

        submitTime  = SectionInfo.SubTime
        self.jobTimesObj.addSubmitTime(JID, submitTime)

        startTime = SectionInfo.StartTime

        numProcessesAdded = 0
        for processNum in SectionInfo.ProcStats.keys():
            state = SectionInfo.ProcStats[processNum]
            processInfoObj = processInfo(sectionID, processNum, procType,
                                         submitTime, state)
            segmentID = sectionID+"."+str(processNum)
            self.processDict[segmentID] = processInfoObj
            if startTime != None:
                endTime = SectionInfo.getProcess(processNum, 0).EndTime
                self.jobTimesObj.addSegment(segmentID, procType, startTime,
                                            endTime)
            numProcessesAdded += 1
            
        return numProcessesAdded


    #
    #

    def updateProcessDict(self, segmentID, processInfoObj):

        try:
            self.processDict[segmentID] = processInfoObj
        except KeyError:
            stderrMessage("SegmentID %s not found" % segmentID)


    #
    #

    def getNumProcInPtypes(self, state, procTypeList=None):

        if procTypeList == None:
            allPtypes = 1
        else:
            allPtypes = 0
            
        numProc = 0
        for segmentID in self.processDict.keys():
            processInfoObj = self.processDict[segmentID]
            procType = processInfoObj.procType
            procState    = processInfoObj.state
            if (allPtypes or procType in procTypeList) and procState == state:
                numProc += 1

        return numProc


    #
    #

    def getNumProcNotInPtypes(self, state, procTypeList):

        numProc = 0
        for segmentID in self.processDict.keys():
            processInfoObj = self.processDict[segmentID]
            procType = processInfoObj.procType
            procState    = processInfoObj.state
            if (procType not in procTypeList) and procState == state:
                numProc += 1

        return numProc


    #
    #

    def writeNumProcessesToLog(self):

        numShortPending = self.getNumProcInPtypes("pending", ["short"])
        numShortRunning = self.getNumProcInPtypes("running", ["short"])

        numMediumPending = self.getNumProcInPtypes("pending", ["medium"])
        numMediumRunning = self.getNumProcInPtypes("running", ["medium"])

        numLongPending = self.getNumProcInPtypes("pending", ["long"])
        numLongRunning = self.getNumProcInPtypes("running", ["long"])

        numGroupsPending = self.getNumProcInPtypes("pending", GROUP_PTYPES)
        numGroupsRunning = self.getNumProcInPtypes("running", GROUP_PTYPES)

        ptypesNotOther = ["short", "medium", "long"]
        for ptype in GROUP_PTYPES:
            ptypesNotOther.append(ptype)
        numOtherPending = self.getNumProcNotInPtypes("pending", ptypesNotOther)
        numOtherRunning = self.getNumProcNotInPtypes("running", ptypesNotOther)

        totalPending = numShortPending + numMediumPending + numLongPending + \
                       + numGroupsPending + numOtherPending

        totalRunning = numShortRunning + numMediumRunning + numLongRunning \
                       + numGroupsRunning + numOtherRunning
        
        stdoutMessage(\
            "Pending (%d): %d %d %d %d %d Running (%d): %d %d %d %d %d" % \
            (totalPending, numShortPending, numMediumPending, numLongPending,
             numGroupsPending, numOtherPending,
             totalRunning, numShortRunning, numMediumRunning, numLongRunning,
             numGroupsRunning, numOtherRunning))

        writeData("pending", [numShortPending, numMediumPending, \
                              numLongPending, numGroupsPending, \
                              numOtherPending])
        writeData("running", [numShortRunning, numMediumRunning, \
                              numLongRunning, numGroupsRunning, \
                              numOtherRunning])
        

    #
    #

    def writeProccessTimeSummaryToLog(self):

        timeListShort, timeListMedium, timeListLong = ([], [], [])
        timeListGroups, timeListOther = ([], [])
        startListShort, startListMedium, startListLong = ([], [], [])
        startListGroups, startListOther = ([], [])
        execListShort, execListMedium, execListLong = ([], [], [])
        execListGroups, execListOther = ([], [])
        JIDList = self.jobTimesObj.getJIDList()
        for JID in JIDList:

            if self.jobTimesObj.getWaitTime(JID) != None:
                procType = self.jobTimesObj.getProcType(JID)
                waitTime = self.jobTimesObj.getWaitTime(JID)
#                stdoutMessage("waitTime: %s %s %s" % (JID, procType, waitTime))
                if procType == "short":
                    timeListShort.append(self.jobTimesObj.getWaitTime(JID))
                if procType == "medium":
                    timeListMedium.append(self.jobTimesObj.getWaitTime(JID))
                if procType == "long":
                    timeListLong.append(self.jobTimesObj.getWaitTime(JID))
                if procType in GROUP_PTYPES:
                    timeListGroups.append(self.jobTimesObj.getWaitTime(JID))
                else:
                    timeListOther.append(self.jobTimesObj.getWaitTime(JID))

            if self.jobTimesObj.getAvgProcessStartTimeDiff(JID) != None:
                procType = self.jobTimesObj.getProcType(JID)
                startTime = self.jobTimesObj.getAvgProcessStartTimeDiff(JID)
#                stdoutMessage("startTime: %s %s %s" % (JID, procType, startTime))
                if procType == "short":
                    startListShort.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                if procType == "medium":
                    startListMedium.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                if procType == "long":
                    startListLong.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                if procType in GROUP_PTYPES:
                    startListGroups.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))
                else:
                    startListOther.append(self.jobTimesObj.getAvgProcessStartTimeDiff(JID))

                    
            if self.jobTimesObj.getAvgProcessExecTime(JID) != None:
                procType = self.jobTimesObj.getProcType(JID)
                execTime = self.jobTimesObj.getAvgProcessExecTime(JID)
#                stdoutMessage("execTime: %s %s %s" % (JID, procType, execTime))
                if procType == "short":
                    execListShort.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                if procType == "medium":
                    execListMedium.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                if procType == "long":
                    execListLong.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                if procType in GROUP_PTYPES:
                    execListGroups.append(self.jobTimesObj.getAvgProcessExecTime(JID))
                else:
                    execListOther.append(self.jobTimesObj.getAvgProcessExecTime(JID))

        waitTimeAvgShort = getAvg(timeListShort)
        waitTimeAvgMedium = getAvg(timeListMedium)
        waitTimeAvgLong = getAvg(timeListLong)
        waitTimeAvgGroups = getAvg(timeListGroups)
        waitTimeAvgOther = getAvg(timeListOther)
        stdoutMessage("waitTimeAvg = %.1f %.1f %.1f %.1f %.1f" % (waitTimeAvgShort, waitTimeAvgMedium, waitTimeAvgLong, waitTimeAvgGroups, waitTimeAvgOther))
        writeData("wait_time", [waitTimeAvgShort, waitTimeAvgMedium, \
                                waitTimeAvgLong, waitTimeAvgGroups, \
                                waitTimeAvgOther])


        startTimeAvgShort = getAvg(startListShort)
        startTimeAvgMedium = getAvg(startListMedium)
        startTimeAvgLong = getAvg(startListLong)
        startTimeAvgGroups = getAvg(startListGroups)
        startTimeAvgOther = getAvg(startListOther)
        stdoutMessage("startTimeDiffAvg = %.1f %.1f %.1f %.1f %.1f" % (startTimeAvgShort, startTimeAvgMedium, startTimeAvgLong, startTimeAvgGroups, startTimeAvgOther))
        writeData("start_time_diff", [startTimeAvgShort, startTimeAvgMedium, \
                                startTimeAvgLong, startTimeAvgGroups, \
                                startTimeAvgOther])
        
        execTimeAvgShort = getAvg(execListShort)
        execTimeAvgMedium = getAvg(execListMedium)
        execTimeAvgLong = getAvg(execListLong)
        execTimeAvgGroups = getAvg(execListGroups)
        execTimeAvgOther = getAvg(execListOther)
        stdoutMessage("execTimeAvg = %.1f %.1f %.1f %.1f %.1f" % (execTimeAvgShort, execTimeAvgMedium, execTimeAvgLong, execTimeAvgGroups, execTimeAvgOther))
        writeData("exec_time", [execTimeAvgShort, execTimeAvgMedium, \
                                execTimeAvgLong, execTimeAvgGroups, \
                                execTimeAvgOther])


    #
    #

    def cout(self):
        for segmentID in self.processDict.keys():
            processInfoObj = self.processDict[segmentID]
            JID, sectionName, procNum = string.split(segmentID, ".")
            startTime = self.jobTimesObj.submitTimeDict[JID]
            stdoutMessage("yo %s %s" % (JID, startTime))
