#!/local/ups/prd/python/v2_1/Linux-2-4/bin/python

# Standard modules
import sys
import time
import os
import string

# FBSNG modules
from class_MyEventListener import MyEventListener

# My modules
from dcParams import *
from dcUtil import *
from class_CafInfo import CafInfo
import shared            #pass module for sharing objects between modules
    

# Main program
#

def main():

    # Create shared CafInfo object
    shared.CafInfoObj = CafInfo()
    

    # Open log file
    #

    fd = os.open(LOG_FILE, os.O_WRONLY | os.O_CREAT | os.O_TRUNC)
    os.dup2(fd, 1)
    os.dup2(fd, 2)
    os.close(fd)


    # Initial polling of section and process information
    #

    stdoutMessage("Initial polling for section list")
    pollStartTime = time.time()
    sectionIDList = shared.CafInfoObj.get()
    timeLastPollCompleted = time.time()
    pollDuration = timeLastPollCompleted - pollStartTime
    stdoutMessage("Polling completed in %d seconds" % pollDuration)


    # Write process accounting information
    #
    
    shared.CafInfoObj.writeNumProcessesToLog()
    shared.CafInfoObj.writeProccessTimeSummaryToLog()
    timeLastDataWriteCompleted = time.time()


    # Initial subscription 
    #

    listener = MyEventListener(sectionIDList)
    numSectionsLast = len(sectionIDList)


    # Loop forever
    #
    
    while 1:

        # Listen to bmgr information for pollingInterval seconds
        #
        
        while ((time.time() - timeLastPollCompleted) < POLLING_INTERVAL):
            nevents = listener.wait(nmax = 1, tmo = 10)


            # Write process accounting information to log file regularly
            # during listen period
            
            if((time.time()-timeLastDataWriteCompleted) > DATA_WRITE_INTERVAL):
                shared.CafInfoObj.writeNumProcessesToLog()
                shared.CafInfoObj.writeProccessTimeSummaryToLog()
                timeLastDataWriteCompleted = time.time()


        # Refresh section subscription
        #

        # Unsubscribe from all currently subscribed sections
        listener.unsubscribeSections()

        # Poll bmgr for new section/process information
        stdoutMessage("Polling for new section list")
        pollStartTime = time.time()
        sectionIDList = shared.CafInfoObj.refresh()
        timeLastPollCompleted = time.time()
        pollDuration = timeLastPollCompleted - pollStartTime
        stdoutMessage("Polling completed in %d seconds" % pollDuration)
        
#        shared.CafInfoObj.cout()

        # Subscribe to section list resulting from poll
        numSubscribed = listener.subscribeSections(sectionIDList)
        netSections = numSubscribed - numSectionsLast
        stdoutMessage(\
            "%5d unsubscribed, %5d subscribed (%d net sections)" % (numSectionsLast, numSubscribed, netSections))
        numSectionsLast = numSubscribed

        # Write process accounting information
        shared.CafInfoObj.writeNumProcessesToLog()
        shared.CafInfoObj.writeProccessTimeSummaryToLog()
        timeLastDataWriteCompleted = time.time()
        

if __name__ == "__main__":

    deamonPid = os.fork()
    if deamonPid > 0:
        sys.exit(0)
    else:
        
        # Set FBSNG config file enviroment. Describes how to contact the bmgr
        #
        os.environ['FBS_CONFIG'] = FBSNG_CFG_FILE

        # Trying to make this deamon process
        os.setsid()
        # Becomes a session leader of a new session
        # Becomes the process group leader of a new process group
        # Has no controlling terminal

        os.chdir(DC_ROOT_DIR)

        os.umask(0)
        # I also need to close all the unnecessary open file descriptors

        main()
