#!/local/ups/prd/python/v2_1/Linux-2-4/bin/python

# batchmon_parser.py
#    Top-level script for parsing of batch monitoring information and
#    subsequent insertion of data into Round Robin Database (RRD) files
#
# [Mark Neubauer 7/15/03]


# Standard modules
import sys
import time
import os
import string
import popen2

# My modules
from bpParams import *
from bpUtil import *


# parser class definition
#

class parser:
    """ Handle parsing of batch monitoring data and insertion into RRD
    """

    # Contructor
    #

    def __init__(self):

        stdoutMessage("Parser daemon started")


    # Method definition - procesDataFile
    #
    
    def processDataFile(self, fileName):
        """ Read data from file and write it into RRD in the appropriate format
        """

        stdoutMessage("Processing %s" % fileName)


        # Try to open5data file and create a file object for it
        #
        
        self.sourceFile = os.path.join(DATA_ROOT_DIR, fileName)
        try:
            self.file = open(self.sourceFile,"r")
        except:
            stderrMessage("error opening file %s" % self.sourceFile)
            return -1


        # From given data file name, extract base name and RRD file name
        #   e.g. pending.dat ==> baseName = "pending" & rrdFile = "pending.rrd"
        #
        
        tmpList = string.split(fileName, ".")
        if len(tmpList) > 1:
            i = string.find(fileName, tmpList[len(tmpList)-1])
            self.baseName = fileName[:i-1]
        else:
            self.baseName = fileName
        self.rrdFile = self.baseName + ".rrd"
        self.rrdFile = os.path.join(RRD_ROOT_DIR, self.rrdFile)


        # If RRD file does not exist, create it
        #
        
        if not os.path.isfile(self.rrdFile):
            self.createRRD()


        # Read data in data file and insert it into RRD file
        # Assumed format is:
        #   time#val1,val2,...,valN
        #
        
        for line in self.file.readlines():
            tmpList = string.split(line, "#")
            if len(tmpList) < 2:
                stderrMessage("bad data format in %s" % self.sourceFile)
            else:
                self.dataTime, dataStr = tmpList
                self.dataList = string.split(dataStr, ",")

            # Insert data into RRD file
            self.updateRRD()


        # Close input data file and remove it
        #
        
        if self.file != None:
            self.file.close()
        stdoutMessage("unlinking file")
        try:
            os.unlink(self.sourceFile)
        except:
            stdoutMessage("not able to unlink the file")


    # Method definition - createRRD
    #
    
    def createRRD(self):
        """ Create empty RRD file
        """

        stdoutMessage("Creating %s" % self.baseName)


        # Assign data source and round-robin archive strings from dictionaries
        #
       
        if self.baseName[:5] == "user_":
            dsStr = dsDict["user"]
            rraStr = rraDict["user"]
        else: 
            dsStr = dsDict[self.baseName]
            rraStr = rraDict[self.baseName]


        # Create RRD file using "rrdtool create" command
        #
        
        startTime = "-86400s"           # allow for entries up to 24hrs prior
        stepTime = 300.                 # feed data in every 5 mins
        command = "%s create %s --start %s --step %s %s %s" % \
               (RRDTOOL_BIN, self.rrdFile, startTime, stepTime, dsStr, rraStr)
        stdoutMessage(command)
        childout, childin, childerr = popen2.popen3(command)
        childin.close()
        tempOut = childout.readlines()
        childout.close()
        tempErr = childerr.readlines()
        childerr.close()

        for line in tempErr:
            line = string.strip(line)
            stderrMessage(line)

        for line in tempOut:
            line = string.strip(line)
            stdoutMessage(line)


    # Method definition - updateRRD
    #

    def updateRRD(self):
        """ Update existing RRD file with new data
        """

        # Create string of colon separated data time and data elements
        #
        
        valStr = self.dataTime
        for dataElement in self.dataList:
            dataElement = string.strip(dataElement)
            if dataElement == "None":
                valStr += ":NaN"
            else:
                valStr += ":%s" % float(dataElement)


        # Insert new data into RRD file using "rrdtool update" command
        #
        
        command = "%s update %s %s" % (RRDTOOL_BIN, self.rrdFile, valStr)
        stdoutMessage(command)
        childout, childin, childerr = popen2.popen3(command)
        childin.close()
        tempOut = childout.readlines()
        childout.close()
        tempErr = childerr.readlines()
        childerr.close()

        for line in tempErr:
            line = string.strip(line)
            stderrMessage(line)

        for line in tempOut:
            line = string.strip(line)
            stdoutMessage(line)


# Main program
#

def main():


    # Open log file
    #

    LOG_FILE = os.path.join(LOG_DIR, "parser.log")
    fd = os.open(LOG_FILE, os.O_WRONLY | os.O_CREAT | os.O_TRUNC)
    os.dup2(fd, 1)
    os.dup2(fd, 2)
    os.close(fd)

    parserObj = parser()
    
    while 1:
        dataFilesToProcess = ["pending.dat",
                              "running.dat",
                              "wait_time.dat",
                              "start_time_diff.dat",
                              "exec_time.dat"]

        fileList = os.listdir(DATA_ROOT_DIR)
        for file in fileList:
            if file[:5] == "user_" and file[-4:] == ".dat":
                dataFilesToProcess.append(file)


        for file in dataFilesToProcess:
            parserObj.processDataFile(file)

        stdoutMessage("hybernating ....")
        time.sleep(PARSER_INTERVAL)


if __name__ == "__main__":

    deamonPid = os.fork()
    if deamonPid > 0:
        sys.exit(0)
    else:
        
        # Trying to make this deamon process
        os.setsid()
        # Becomes a session leader of a new session
        # Becomes the process group leader of a new process group
        # Has no controlling terminal

        os.chdir("/")

        os.umask(0)
        # I also need to close all the unnecessary open file descriptors

        main()


